@@ -1481,6 +1481,70 @@ userspace_report_debug_info(struct drgn_debug_info_load_state *load)
 	return NULL;
 }
 
+static int should_apply_relocation_section(Elf *elf, size_t shstrndx,
+					   const GElf_Shdr *shdr)
+{
+	if (shdr->sh_type != SHT_RELA && shdr->sh_type != SHT_REL)
+		return 0;
+
+	const char *scnname = elf_strptr(elf, shstrndx, shdr->sh_name);
+	if (!scnname)
+		return -1;
+	if (shdr->sh_type == SHT_RELA) {
+		if (!strstartswith(scnname, ".rela."))
+			return 0;
+		scnname += sizeof(".rela.") - 1;
+	} else {
+		if (!strstartswith(scnname, ".rel."))
+			return 0;
+		scnname += sizeof(".rel.") - 1;
+	}
+	return (strstartswith(scnname, "debug_") ||
+		strstartswith(scnname, "orc_"));
+}
+
+static inline struct drgn_error *get_reloc_sym_value(const void *syms,
+						     size_t num_syms,
+						     const uint64_t *sh_addrs,
+						     size_t shdrnum,
+						     bool is_64_bit,
+						     bool bswap,
+						     uint32_t r_sym,
+						     uint64_t *ret)
+{
+	if (r_sym >= num_syms) {
+		return drgn_error_create(DRGN_ERROR_OTHER,
+					 "invalid ELF relocation symbol");
+	}
+	uint16_t st_shndx;
+	uint64_t st_value;
+	if (is_64_bit) {
+		const Elf64_Sym *sym = (Elf64_Sym *)syms + r_sym;
+		memcpy(&st_shndx, &sym->st_shndx, sizeof(st_shndx));
+		memcpy(&st_value, &sym->st_value, sizeof(st_value));
+		if (bswap) {
+			st_shndx = bswap_16(st_shndx);
+			st_value = bswap_64(st_value);
+		}
+	} else {
+		const Elf32_Sym *sym = (Elf32_Sym *)syms + r_sym;
+		memcpy(&st_shndx, &sym->st_shndx, sizeof(st_shndx));
+		uint32_t st_value32;
+		memcpy(&st_value32, &sym->st_value, sizeof(st_value32));
+		if (bswap) {
+			st_shndx = bswap_16(st_shndx);
+			st_value32 = bswap_32(st_value32);
+		}
+		st_value = st_value32;
+	}
+	if (st_shndx >= shdrnum) {
+		return drgn_error_create(DRGN_ERROR_OTHER,
+					 "invalid ELF symbol section index");
+	}
+	*ret = sh_addrs[st_shndx] + st_value;
+	return NULL;
+}
+
 static struct drgn_error *
 apply_elf_relas(const struct drgn_relocating_section *relocating,
 		Elf_Data *reloc_data, Elf_Data *symtab_data,
@@ -1491,7 +1555,7 @@ apply_elf_relas(const struct drgn_relocating_section *relocating,
 
 	bool is_64_bit = drgn_platform_is_64_bit(platform);
 	bool bswap = drgn_platform_bswap(platform);
-	apply_elf_rela_fn *apply_elf_rela = platform->arch->apply_elf_rela;
+	apply_elf_reloc_fn *apply_elf_reloc = platform->arch->apply_elf_reloc;
 
 	const void *relocs = reloc_data->d_buf;
 	size_t reloc_size = is_64_bit ? sizeof(Elf64_Rela) : sizeof(Elf32_Rela);
@@ -1537,38 +1601,77 @@ apply_elf_relas(const struct drgn_relocating_section *relocating,
 			r_type = ELF32_R_TYPE(r_info32);
 			r_addend = r_addend32;
 		}
-		if (r_sym >= num_syms) {
-			return drgn_error_create(DRGN_ERROR_OTHER,
-						 "invalid ELF relocation symbol");
-		}
-		uint16_t st_shndx;
-		uint64_t st_value;
-		if (is_64_bit) {
-			const Elf64_Sym *sym = (Elf64_Sym *)syms + r_sym;
-			memcpy(&st_shndx, &sym->st_shndx, sizeof(st_shndx));
-			memcpy(&st_value, &sym->st_value, sizeof(st_value));
-			if (bswap) {
-				st_shndx = bswap_16(st_shndx);
-				st_value = bswap_64(st_value);
-			}
-		} else {
-			const Elf32_Sym *sym = (Elf32_Sym *)syms + r_sym;
-			memcpy(&st_shndx, &sym->st_shndx, sizeof(st_shndx));
-			uint32_t st_value32;
-			memcpy(&st_value32, &sym->st_value, sizeof(st_value32));
-			if (bswap) {
-				st_shndx = bswap_16(st_shndx);
-				st_value32 = bswap_32(st_value32);
-			}
-			st_value = st_value32;
-		}
-		if (st_shndx >= shdrnum) {
-			return drgn_error_create(DRGN_ERROR_OTHER,
-						 "invalid ELF symbol section index");
-		}
+		uint64_t sym_value;
+		err = get_reloc_sym_value(syms, num_syms, sh_addrs, shdrnum,
+					  is_64_bit, bswap, r_sym, &sym_value);
+		if (err)
+			return err;
 
-		err = apply_elf_rela(relocating, r_offset, r_type, r_addend,
-				     sh_addrs[st_shndx] + st_value);
+		err = apply_elf_reloc(relocating, r_offset, r_type, &r_addend,
+				      sym_value);
+		if (err)
+			return err;
+	}
+	return NULL;
+}
+
+static struct drgn_error *
+apply_elf_rels(const struct drgn_relocating_section *relocating,
+	       Elf_Data *reloc_data, Elf_Data *symtab_data,
+	       const uint64_t *sh_addrs, size_t shdrnum,
+	       const struct drgn_platform *platform)
+{
+	struct drgn_error *err;
+
+	bool is_64_bit = drgn_platform_is_64_bit(platform);
+	bool bswap = drgn_platform_bswap(platform);
+	apply_elf_reloc_fn *apply_elf_reloc = platform->arch->apply_elf_reloc;
+
+	const void *relocs = reloc_data->d_buf;
+	size_t reloc_size = is_64_bit ? sizeof(Elf64_Rel) : sizeof(Elf32_Rel);
+	size_t num_relocs = reloc_data->d_size / reloc_size;
+
+	const void *syms = symtab_data->d_buf;
+	size_t sym_size = is_64_bit ? sizeof(Elf64_Sym) : sizeof(Elf32_Sym);
+	size_t num_syms = symtab_data->d_size / sym_size;
+
+	for (size_t i = 0; i < num_relocs; i++) {
+		uint64_t r_offset;
+		uint32_t r_sym;
+		uint32_t r_type;
+		if (is_64_bit) {
+			const Elf64_Rel *rel = (Elf64_Rel *)relocs + i;
+			uint64_t r_info;
+			memcpy(&r_offset, &rel->r_offset, sizeof(r_offset));
+			memcpy(&r_info, &rel->r_info, sizeof(r_info));
+			if (bswap) {
+				r_offset = bswap_64(r_offset);
+				r_info = bswap_64(r_info);
+			}
+			r_sym = ELF64_R_SYM(r_info);
+			r_type = ELF64_R_TYPE(r_info);
+		} else {
+			const Elf32_Rel *rel32 = (Elf32_Rel *)relocs + i;
+			uint32_t r_offset32;
+			uint32_t r_info32;
+			memcpy(&r_offset32, &rel32->r_offset, sizeof(r_offset32));
+			memcpy(&r_info32, &rel32->r_info, sizeof(r_info32));
+			if (bswap) {
+				r_offset32 = bswap_32(r_offset32);
+				r_info32 = bswap_32(r_info32);
+			}
+			r_offset = r_offset32;
+			r_sym = ELF32_R_SYM(r_info32);
+			r_type = ELF32_R_TYPE(r_info32);
+		}
+		uint64_t sym_value;
+		err = get_reloc_sym_value(syms, num_syms, sh_addrs, shdrnum,
+					  is_64_bit, bswap, r_sym, &sym_value);
+		if (err)
+			return err;
+
+		err = apply_elf_reloc(relocating, r_offset, r_type, NULL,
+				      sym_value);
 		if (err)
 			return err;
 	}
@@ -1597,7 +1700,7 @@ static struct drgn_error *relocate_elf_file(Elf *elf)
 
 	struct drgn_platform platform;
 	drgn_platform_from_elf(ehdr, &platform);
-	if (!platform.arch->apply_elf_rela) {
+	if (!platform.arch->apply_elf_reloc) {
 		/* Unsupported; fall back to libdwfl. */
 		return NULL;
 	}
@@ -1638,15 +1741,13 @@ static struct drgn_error *relocate_elf_file(Elf *elf)
 		if (reloc_shdr->sh_type != SHT_RELA)
 			continue;
 
-		const char *scnname = elf_strptr(elf, shstrndx,
-						 reloc_shdr->sh_name);
-		if (!scnname) {
+		int r = should_apply_relocation_section(elf, shstrndx,
+							reloc_shdr);
+		if (r < 0) {
 			err = drgn_error_libelf();
 			goto out;
 		}
-
-		if (strstartswith(scnname, ".rela.debug_") ||
-		    strstartswith(scnname, ".rela.orc_")) {
+		if (r) {
 			Elf_Scn *scn = elf_getscn(elf, reloc_shdr->sh_info);
 			if (!scn) {
 				err = drgn_error_libelf();
@@ -1691,9 +1792,15 @@ static struct drgn_error *relocate_elf_file(Elf *elf)
 				.bswap = drgn_platform_bswap(&platform),
 			};
 
-			err = apply_elf_relas(&relocating, reloc_data,
-					      symtab_data, sh_addrs, shdrnum,
-					      &platform);
+			if (reloc_shdr->sh_type == SHT_RELA) {
+				err = apply_elf_relas(&relocating, reloc_data,
+						      symtab_data, sh_addrs,
+						      shdrnum, &platform);
+			} else {
+				err = apply_elf_rels(&relocating, reloc_data,
+						     symtab_data, sh_addrs,
+						     shdrnum, &platform);
+			}
 			if (err)
 				goto out;
 
