@@ -36,7 +36,7 @@
 /** Minor version of drgn. */
 #define DRGN_VERSION_MINOR 0
 /** Patch level of drgn. */
-#define DRGN_VERSION_PATCH 17
+#define DRGN_VERSION_PATCH 19
 
 /**
  * @defgroup ErrorHandling Error handling
@@ -81,6 +81,8 @@ enum drgn_error_code {
 	DRGN_ERROR_OUT_OF_BOUNDS,
 	/** Operation was attempted with absent object. */
 	DRGN_ERROR_OBJECT_ABSENT,
+	/** Functionality is not implemented. */
+	DRGN_ERROR_NOT_IMPLEMENTED,
 	/** Number of defined error codes. */
 	DRGN_NUM_ERROR_CODES,
 } __attribute__((__packed__));
@@ -361,7 +363,12 @@ drgn_type_language(struct drgn_type *type);
 enum drgn_architecture {
 	DRGN_ARCH_UNKNOWN,
 	DRGN_ARCH_X86_64,
+	DRGN_ARCH_I386,
+	DRGN_ARCH_AARCH64,
+	DRGN_ARCH_ARM,
 	DRGN_ARCH_PPC64,
+	DRGN_ARCH_RISCV64,
+	DRGN_ARCH_RISCV32,
 };
 
 /** Flags describing a @ref drgn_platform. */
