@@ -27,8 +27,15 @@ static struct drgn_error *drgn_platform_from_kdump(kdump_ctx_t *ctx,
 	}
 	if (strcmp(str, KDUMP_ARCH_X86_64) == 0)
 		arch = &arch_info_x86_64;
+	else if (strcmp(str, KDUMP_ARCH_IA32) == 0)
+		arch = &arch_info_i386;
+	else if (strcmp(str, KDUMP_ARCH_AARCH64) == 0)
+		arch = &arch_info_aarch64;
+	else if (strcmp(str, KDUMP_ARCH_ARM) == 0)
+		arch = &arch_info_arm;
 	else if (strcmp(str, KDUMP_ARCH_PPC64) == 0)
 		arch = &arch_info_ppc64;
+	/* libkdumpfile doesn't support RISC-V */
 	else
 		arch = &arch_info_unknown;
 
