@@ -17,23 +17,43 @@
 /**
  * @ingroup Internals
  *
- * @defgroup Languages Languages
+ * @defgroup LanguageInternals Languages
  *
  * Language support.
  *
  * This defines the interface which support for a language must implement,
  * including operators and parsing.
  *
+ * To add a new language:
+ * - Add a @ref drgn_language_number for it.
+ * - Define a @ref drgn_language for it, and set @ref drgn_language::number to
+ *   the corresponding @ref drgn_language_number.
+ * - Add it to drgn.h.
+ * - Add it to @ref drgn_languages.
+ * - Add it to add_languages() in the Python bindings.
+ * - Add it to _drgn.pyi.
  * @{
  */
 
+/**
+ * Language numbers.
+ *
+ * These can be used as indices for storing language-specific data in an array.
+ */
+enum drgn_language_number {
+	DRGN_LANGUAGE_C,
+	DRGN_LANGUAGE_CPP,
+	DRGN_NUM_LANGUAGES,
+};
+
 typedef struct drgn_error *drgn_format_type_fn(struct drgn_qualified_type,
 					       char **);
 typedef struct drgn_error *drgn_format_object_fn(const struct drgn_object *,
 						 size_t,
 						 enum drgn_format_object_flags,
 						 char **);
-typedef struct drgn_error *drgn_find_type_fn(struct drgn_program *prog,
+typedef struct drgn_error *drgn_find_type_fn(const struct drgn_language *lang,
+					     struct drgn_program *prog,
 					     const char *name,
 					     const char *filename,
 					     struct drgn_qualified_type *ret);
@@ -66,6 +86,8 @@ typedef struct drgn_error *drgn_cmp_op(const struct drgn_object *lhs,
 struct drgn_language {
 	/** Name of this programming language. */
 	const char *name;
+	/** Number of this programming language. */
+	enum drgn_language_number number;
 	/** Implement @ref drgn_format_type_name(). */
 	drgn_format_type_fn *format_type_name;
 	/** Implement @ref drgn_format_type(). */
@@ -126,41 +148,8 @@ struct drgn_language {
 	drgn_unary_op *op_not;
 };
 
-drgn_format_type_fn c_format_type_name;
-drgn_format_type_fn c_format_type;
-drgn_format_object_fn c_format_object;
-drgn_find_type_fn c_find_type;
-drgn_bit_offset_fn c_bit_offset;
-drgn_integer_literal_fn c_integer_literal;
-drgn_bool_literal_fn c_bool_literal;
-drgn_float_literal_fn c_float_literal;
-drgn_cast_op c_op_cast;
-drgn_bool_op c_op_bool;
-drgn_cmp_op c_op_cmp;
-drgn_binary_op c_op_add;
-drgn_binary_op c_op_sub;
-drgn_binary_op c_op_mul;
-drgn_binary_op c_op_div;
-drgn_binary_op c_op_mod;
-drgn_binary_op c_op_lshift;
-drgn_binary_op c_op_rshift;
-drgn_binary_op c_op_and;
-drgn_binary_op c_op_or;
-drgn_binary_op c_op_xor;
-drgn_unary_op c_op_pos;
-drgn_unary_op c_op_neg;
-drgn_unary_op c_op_not;
-
-enum {
-	DRGN_LANGUAGE_C,
-	DRGN_LANGUAGE_CPP,
-	DRGN_NUM_LANGUAGES,
-};
-
-extern const struct drgn_language drgn_languages[DRGN_NUM_LANGUAGES];
-
-#define drgn_language_c drgn_languages[DRGN_LANGUAGE_C]
-#define drgn_language_cpp drgn_languages[DRGN_LANGUAGE_CPP]
+/** Mapping from @ref drgn_language_number to @ref drgn_language. */
+extern const struct drgn_language * const drgn_languages[];
 
 /** Language to be used when actual language is unknown. */
 #define drgn_default_language drgn_language_c
