@@ -1,12 +1,10 @@
 # Copyright (c) Meta Platforms, Inc. and affiliates.
 # SPDX-License-Identifier: GPL-3.0-or-later
-
-import unittest
-
+from tests import TestCase
 from tests.libdrgn import Lexer, drgn_test_lexer_func
 
 
-class TestLexer(unittest.TestCase):
+class TestLexer(TestCase):
     def test_pop(self):
         lexer = Lexer(drgn_test_lexer_func, "12345")
         for i in range(5):
