@@ -9,7 +9,16 @@ import os
 from pathlib import Path
 import re
 import sys
-from typing import AsyncIterator, Dict, List, NamedTuple, Optional, Sequence, Union
+from typing import (
+    AsyncIterator,
+    Dict,
+    List,
+    NamedTuple,
+    Optional,
+    Sequence,
+    Tuple,
+    Union,
+)
 
 import aiohttp
 import uritemplate
@@ -18,7 +27,7 @@ from util import KernelVersion
 from vmtest.asynciosubprocess import check_call, check_output
 from vmtest.download import VMTEST_GITHUB_RELEASE, available_kernel_releases
 from vmtest.githubapi import AioGitHubApi
-from vmtest.kbuild import KERNEL_LOCALVERSION, KBuild
+from vmtest.kbuild import KERNEL_FLAVORS, KBuild, KernelFlavor
 
 logger = logging.getLogger(__name__)
 
@@ -66,7 +75,7 @@ async def get_latest_kernel_tags() -> List[str]:
     return ["v" + str(version) for version in sorted(latest.values(), reverse=True)]
 
 
-def kernel_tag_to_release(tag: str) -> str:
+def kernel_tag_to_release(tag: str, flavor: KernelFlavor) -> str:
     match = re.fullmatch(r"v([0-9]+\.[0-9]+)(\.[0-9]+)?(-rc\d+)?", tag)
     assert match
     return "".join(
@@ -74,7 +83,7 @@ def kernel_tag_to_release(tag: str) -> str:
             match.group(1),
             match.group(2) or ".0",
             match.group(3) or "",
-            KERNEL_LOCALVERSION,
+            flavor.localversion(),
         ]
     )
 
@@ -111,17 +120,31 @@ async def fetch_kernel_tags(kernel_dir: Path, kernel_tags: Sequence[str]) -> Non
 
 
 async def build_kernels(
-    kernel_dir: Path, build_dir: Path, arch: str, kernel_revs: Sequence[str]
+    kernel_dir: Path,
+    build_dir: Path,
+    arch: str,
+    kernel_revs: Sequence[Tuple[str, Sequence[KernelFlavor]]],
+    keep_builds: bool,
 ) -> AsyncIterator[Path]:
     build_dir.mkdir(parents=True, exist_ok=True)
-    for rev in kernel_revs:
-        rev_build_dir = build_dir / ("build-" + rev)
-        logger.info("checking out %s in %s", rev, rev_build_dir)
+    for rev, flavors in kernel_revs:
+        logger.info("checking out %s in %s", rev, kernel_dir)
         await check_call("git", "-C", str(kernel_dir), "checkout", "-q", rev)
-        with open(build_dir / f"build-{rev}.log", "w") as build_log_file:
-            kbuild = KBuild(kernel_dir, rev_build_dir, arch, build_log_file)
-            await kbuild.build()
-            yield await kbuild.package(build_dir)
+        for flavor in flavors:
+            flavor_rev_build_dir = build_dir / f"build-{flavor.name}-{rev}"
+            with open(
+                build_dir / f"build-{flavor.name}-{rev}.log", "w"
+            ) as build_log_file:
+                kbuild = KBuild(
+                    kernel_dir, flavor_rev_build_dir, flavor, arch, build_log_file
+                )
+                await kbuild.build()
+                yield await kbuild.package(build_dir)
+                if not keep_builds:
+                    logger.info("deleting %s", flavor_rev_build_dir)
+                    # Shell out instead of using, e.g., shutil.rmtree(), to
+                    # avoid blocking the main thread and the GIL.
+                    await check_call("rm", "-rf", str(flavor_rev_build_dir))
 
 
 class AssetUploadWork(NamedTuple):
@@ -174,8 +197,15 @@ async def main() -> None:
         help="build and upload latest supported kernel releases",
     )
     parser.add_argument(
-        "--dry-run",
-        action="store_true",
+        "--no-build",
+        dest="build",
+        action="store_false",
+        help="don't build or upload anything; just log what would be built",
+    )
+    parser.add_argument(
+        "--no-upload",
+        dest="upload",
+        action="store_false",
         help="build but don't upload anything to GitHub",
     )
     parser.add_argument(
@@ -192,6 +222,19 @@ async def main() -> None:
         help="directory for build artifacts",
         default=".",
     )
+    parser.add_argument(
+        "--no-keep-builds",
+        dest="keep_builds",
+        action="store_false",
+        help="delete kernel builds after packaging (default if uploading)",
+        default=argparse.SUPPRESS,
+    )
+    parser.add_argument(
+        "--keep-builds",
+        action="store_true",
+        help="keep kernel builds after packaging (default if not uploading)",
+        default=argparse.SUPPRESS,
+    )
     parser.add_argument(
         "--cache-directory",
         metavar="DIR",
@@ -201,11 +244,16 @@ async def main() -> None:
     )
     args = parser.parse_args()
 
+    if not args.build:
+        args.upload = False
+    if not hasattr(args, "keep_builds"):
+        args.keep_builds = not args.upload
+
     arch = "x86_64"
 
     async with aiohttp.ClientSession(trust_env=True) as session:
         GITHUB_TOKEN = os.getenv("GITHUB_TOKEN")
-        if GITHUB_TOKEN is None and not args.dry_run:
+        if GITHUB_TOKEN is None and args.upload:
             sys.exit("GITHUB_TOKEN environment variable is not set")
         gh = AioGitHubApi(session, GITHUB_TOKEN)
 
@@ -227,47 +275,63 @@ async def main() -> None:
             ", ".join(sorted(kernel_releases, key=KernelVersion, reverse=True)),
         )
 
+        to_build = []
         if args.latest_kernels:
             logger.info("latest kernel versions: %s", ", ".join(latest_kernel_tags))
-            kernel_tags = [
-                tag
-                for tag in latest_kernel_tags
-                if kernel_tag_to_release(tag) not in kernel_releases
-            ]
-        else:
-            kernel_tags = []
+            for tag in latest_kernel_tags:
+                flavors = [
+                    flavor
+                    for flavor in KERNEL_FLAVORS
+                    if kernel_tag_to_release(tag, flavor) not in kernel_releases
+                ]
+                if flavors:
+                    to_build.append((tag, flavors))
 
-        if kernel_tags:
-            logger.info("kernel versions to build: %s", ", ".join(kernel_tags))
-
-            if not args.dry_run:
-                upload_queue: "asyncio.Queue[Optional[AssetUploadWork]]" = (
-                    asyncio.Queue()
-                )
-                uploader = asyncio.create_task(asset_uploader(gh, upload_queue))
-
-            await fetch_kernel_tags(args.kernel_directory, kernel_tags)
-
-            async for kernel_package in build_kernels(
-                args.kernel_directory, args.build_directory, arch, kernel_tags
-            ):
-                if args.dry_run:
-                    logger.info("would upload %s", kernel_package)
-                else:
-                    await upload_queue.put(
-                        AssetUploadWork(
-                            upload_url=github_release["upload_url"],
-                            path=kernel_package,
-                            name=kernel_package.name,
-                            content_type="application/zstd",
-                        )
+        if to_build:
+            logger.info(
+                "kernel versions to build: %s",
+                ", ".join(
+                    f"{tag} ({', '.join([flavor.name for flavor in flavors])})"
+                    for tag, flavors in to_build
+                ),
+            )
+            if args.build:
+                if args.upload:
+                    upload_queue: "asyncio.Queue[Optional[AssetUploadWork]]" = (
+                        asyncio.Queue()
                     )
+                    uploader = asyncio.create_task(asset_uploader(gh, upload_queue))
 
-            if not args.dry_run:
-                await upload_queue.put(None)
-                await upload_queue.join()
-                if not await uploader:
-                    sys.exit("some uploads failed")
+                await fetch_kernel_tags(
+                    args.kernel_directory, [tag for tag, _ in to_build]
+                )
+
+                async for kernel_package in build_kernels(
+                    args.kernel_directory,
+                    args.build_directory,
+                    arch,
+                    to_build,
+                    args.keep_builds,
+                ):
+                    if args.upload:
+                        await upload_queue.put(
+                            AssetUploadWork(
+                                upload_url=github_release["upload_url"],
+                                path=kernel_package,
+                                name=kernel_package.name,
+                                content_type="application/zstd",
+                            )
+                        )
+                    else:
+                        logger.info("would upload %s", kernel_package)
+
+                if args.upload:
+                    await upload_queue.put(None)
+                    await upload_queue.join()
+                    if not await uploader:
+                        sys.exit("some uploads failed")
+        else:
+            logger.info("nothing to build")
 
 
 if __name__ == "__main__":
