@@ -24,6 +24,8 @@ _INIT_TEMPLATE = r"""#!{busybox} sh
 set -eu
 
 export BUSYBOX={busybox}
+export PYTHON={python}
+{kdump_needs_nosmp}
 
 trap '"$BUSYBOX" poweroff -f' EXIT
 
@@ -57,6 +59,8 @@ cd /
 
 # Mount additional filesystems.
 "$BUSYBOX" mount -t devtmpfs -o nosuid,noexec dev /dev
+"$BUSYBOX" mkdir /dev/shm
+"$BUSYBOX" mount -t tmpfs -o nosuid,nodev tmpfs /dev/shm
 "$BUSYBOX" mount -t proc -o nosuid,nodev,noexec proc /proc
 "$BUSYBOX" mount -t sysfs -o nosuid,nodev,noexec sys /sys
 # cgroup2 was added in Linux v4.5.
@@ -212,7 +216,9 @@ def run_in_vm(command: str, kernel_dir: Path, build_dir: Path) -> int:
                 _INIT_TEMPLATE.format(
                     _9PFS_MSIZE=_9PFS_MSIZE,
                     busybox=shlex.quote(busybox),
+                    python=shlex.quote(sys.executable),
                     command=shlex.quote(command),
+                    kdump_needs_nosmp="" if kvm_args else "export KDUMP_NEEDS_NOSMP=1",
                 )
             )
         os.chmod(init, 0o755)
@@ -244,7 +250,7 @@ def run_in_vm(command: str, kernel_dir: Path, build_dir: Path) -> int:
 
                 "-kernel", str(kernel_dir / "vmlinuz"),
                 "-append",
-                f"rootfstype=9p rootflags=trans=virtio,cache=loose,msize={_9PFS_MSIZE} ro console=0,115200 panic=-1 init={init}",
+                f"rootfstype=9p rootflags=trans=virtio,cache=loose,msize={_9PFS_MSIZE} ro console=0,115200 panic=-1 crashkernel=256M init={init}",
                 # fmt: on
             ],
             env=env,
@@ -277,6 +283,11 @@ def run_in_vm(command: str, kernel_dir: Path, build_dir: Path) -> int:
 
 if __name__ == "__main__":
     import argparse
+    import logging
+
+    logging.basicConfig(
+        format="%(asctime)s:%(levelname)s:%(name)s:%(message)s", level=logging.INFO
+    )
 
     parser = argparse.ArgumentParser(
         description="run vmtest virtual machine",
